<?php

namespace Drupal\k8s\Service;

use Drupal\Core\Entity\ContentEntityInterface;

/**
 * The Interface for K8sOperationsService.
 */
interface K8sOperationsServiceInterface {

  /**
   * Update K8s Namespace.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The K8s Pod entity.
   * @param array $params
   *   Optional parameters array.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function updateNamespace(ContentEntityInterface $entity, array $params): bool;

  /**
   * Update K8s Schedule.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The K8s Schedule entity.
   * @param array $params
   *   Optional parameters array.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function updateSchedule(ContentEntityInterface $entity, array $params): bool;

  /**
   * Update K8s Entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The K8s entity.
   * @param array $params
   *   Optional parameters array.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function updateEntity(ContentEntityInterface $entity, array $params): bool;

  /**
   * Create a cronjob that gets a yaml and schedules a pod.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to be created.
   * @param string $cloud_context
   *   The cloud context.
   * @param string $namespace
   *   The namespace.
   * @param string $yaml_url
   *   The YAML URL of the pod to start.
   * @param int $start_hour
   *   The hour of the schedule.
   * @param int $start_minute
   *   The minute of the schedule.
   * @param int $stop_hour
   *   The hour of the schedule.
   * @param int $stop_minute
   *   The minute of the schedule.
   */
  public function createCronJobFromYaml(
    ContentEntityInterface $entity,
    string $cloud_context,
    string $namespace,
    string $yaml_url,
    int $start_hour,
    int $start_minute,
    int $stop_hour,
    int $stop_minute
  ): bool;

  /**
   * Create K8s Pod.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The K8s Schedule entity.
   * @param string $cloud_context
   *   The cloud context.
   * @param string $namespace
   *   The namespace.
   * @param string $detail
   *   Detail data for K8s entity.
   * @param bool $enable_time_scheduler
   *   Enable Time Scheduler flag.
   * @param string $time_scheduler_option
   *   Time scheduler option.
   * @param string $yaml_url
   *   The YAML URL of the pod to start.
   * @param int $startHour
   *   The start hour for K8s Schedule.
   * @param int $startMinute
   *   The start minute for K8s Schedule.
   * @param int $stopHour
   *   The stop hour for K8s Schedule.
   * @param int $stopMinute
   *   The stop minute for K8s Schedule.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createPod(
    ContentEntityInterface $entity,
    string $cloud_context,
    string $namespace,
    string $detail,
    bool $enable_time_scheduler,
    string $time_scheduler_option,
    string $yaml_url,
    int $startHour,
    int $startMinute,
    int $stopHour,
    int $stopMinute
  ): bool;

  /**
   * Create K8s Namespace.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to be created.
   * @param string $cloud_context
   *   The cloud context.
   * @param string $name
   *   The name of the Namespace.
   * @param array $labels
   *   label data for K8s Namespace.
   */
  public function createNamespace(
    ContentEntityInterface $entity,
    string $cloud_context,
    string $name,
    array $labels
  ): bool;

  /**
   * Create K8s Deployment.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The K8s Schedule entity.
   * @param string $cloud_context
   *   The cloud context.
   * @param string $namespace
   *   The namespace.
   * @param string $detail
   *   Detail data for K8s entity.
   * @param bool $enable_time_scheduler
   *   Enable Time Scheduler flag.
   * @param string $time_scheduler_option
   *   Time scheduler option.
   * @param string $yaml_url
   *   The YAML URL of the pod to start.
   * @param int $startHour
   *   The start hour for K8s Schedule.
   * @param int $startMinute
   *   The start minute for K8s Schedule.
   * @param int $stopHour
   *   The stop hour for K8s Schedule.
   * @param int $stopMinute
   *   The stop minute for K8s Schedule.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createDeployment(
    ContentEntityInterface $entity,
    string $cloud_context,
    string $namespace,
    string $detail,
    bool $enable_time_scheduler,
    string $time_scheduler_option,
    string $yaml_url,
    int $startHour,
    int $startMinute,
    int $stopHour,
    int $stopMinute
  ): bool;

  /**
   * Create K8s other entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The K8s Schedule entity.
   * @param string $cloud_context
   *   The cloud context.
   * @param string $namespace
   *   The namespace.
   * @param string $detail
   *   Detail data for K8s entity.
   * @param bool $enable_time_scheduler
   *   Enable Time Scheduler flag.
   * @param int $startHour
   *   The start hour for K8s Schedule.
   * @param int $startMinute
   *   The start minute for K8s Schedule.
   * @param int $stopHour
   *   The stop hour for K8s Schedule.
   * @param int $stopMinute
   *   The stop minute for K8s Schedule.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createK8sEntity(
    ContentEntityInterface $entity,
    string $cloud_context,
    string $namespace,
    string $detail,
    bool $enable_time_scheduler,
    int $startHour,
    int $startMinute,
    int $stopHour,
    int $stopMinute
  ): bool;

  /**
   * Scale K8s Deployment.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to be created.
   * @param int $pod_number
   *   Desire number of pods.
   */
  public function scaleDeployment(
    ContentEntityInterface $entity,
    int $pod_number
  ): bool;

  /**
   * Delete K8s Namespace.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to be created.
   */
  public function deleteNamespace(ContentEntityInterface $entity): bool;

  /**
   * Delete K8s entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to be deleted.
   */
  public function deleteK8sEntity(ContentEntityInterface $entity): bool;

}
