<?php

namespace Drupal\openstack\Service\Ec2;

use Drupal\aws_cloud\Entity\Ec2\ImageInterface;
use Drupal\aws_cloud\Service\Ec2\Ec2OperationsService;
use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\cloud\Entity\CloudConfigInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Service\CloudServiceInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\openstack\Service\OpenStackServiceFactoryInterface;
use Drupal\openstack\Service\Rest\OpenStackService as OpenStackRestService;

/**
 * Entity update methods for REST API processing.
 */
class OpenStackOperationsService extends Ec2OperationsService implements OpenStackOperationsServiceInterface {

  /**
   * The cloud service provider plugin manager (CloudConfigPluginManager).
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  protected $cloudConfigPluginManager;

  /**
   * The Cloud service.
   *
   * @var \Drupal\cloud\Service\CloudServiceInterface
   */
  protected $cloudService;

  /**
   * The entity.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $entity;

  /**
   * The OpenStack Service Factory.
   *
   * @var \Drupal\openstack\Service\OpenStackServiceFactoryInterface
   */
  protected $openStackServiceFactory;

  /**
   * Ec2OperationsService constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   The AWS Cloud EC2 service.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\cloud\Service\CloudServiceInterface $cloud_service
   *   The Cloud service.
   * @param \Drupal\openstack\Service\OpenStackServiceFactoryInterface $openstack_service_factory
   *   Object for interfacing with OpenStack Service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    Ec2ServiceInterface $ec2_service,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    CloudServiceInterface $cloud_service,
    OpenStackServiceFactoryInterface $openstack_service_factory
  ) {
    parent::__construct($entity_type_manager, $ec2_service);

    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->cloudService = $cloud_service;
    $this->openStackServiceFactory = $openstack_service_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function setEc2Service(mixed $ec2_service): void {
    $this->ec2Service = $ec2_service;
  }

  /**
   * {@inheritdoc}
   */
  public function setEntity(EntityInterface $entity): void {
    $this->entity = $entity;
  }

  /**
   * Check if current cloud config is remote or not.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @return bool
   *   True if the current cloud config is remote.
   */
  private function isCloudConfigRemote(EntityInterface $entity): bool {
    $this->cloudConfigPluginManager->setCloudContext($entity->getCloudContext());
    $cloud_config = $this->cloudConfigPluginManager->loadConfigEntity();
    return $cloud_config->isRemote();
  }

  /**
   * Set the tags for the entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $resource_id
   *   The resource ID.  For example, instance_id, volume_id.
   * @param array $tag_map
   *   The map of tags.
   * @param string $entity_type
   *   The entity type.
   */
  private function setTagsInAws(EntityInterface $entity, string $resource_id, array $tag_map, string $entity_type = ''): void {
    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $tags = [];
    foreach ($tag_map ?: [] as $key => $value) {
      $tags[] = [
        'Key' => $key,
        'Value' => $value,
      ];
    }

    if (!method_exists($this->ec2Service, 'createTags')) {
      return;
    }

    // Create Tags with different parameters for AWS and OpenStack.
    if (preg_match('[^aws_cloud]', $entity->getEntityTypeId()) === 1) {
      $this->ec2Service->createTags([
        'Resources' => [$resource_id],
        'Tags' => $tags,
      ]);
    }
    else {
      $this->ec2Service->createTags([
        'Resources' => [$resource_id],
        'Tags' => $tags,
        'EntityType' => $entity_type,
      ]);
    }
  }

  /**
   * Update name and created_by tags.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $aws_cloud_resource_id
   *   The AWS Cloud resource ID.
   */
  private function updateNameAndCreatedByTags(EntityInterface $entity, string $aws_cloud_resource_id): void {
    $cloud_context = $entity->getCloudContext();
    if ($this->isCloudConfigRemote($entity)) {
      // Get original cloud context.
      if (($pos = strrpos($cloud_context, '_')) !== FALSE) {
        $cloud_context = substr($cloud_context, 0, $pos);
      }
    }

    $bundle = strpos($entity->bundle(), 'aws_cloud') !== FALSE
      ? 'aws_cloud' : 'openstack';
    $uid_key_name = $this->cloudService->getTagCreatedByUid($bundle, $cloud_context);
    $this->setTagsInAws(
      $entity,
      $aws_cloud_resource_id,
      [
        $entity->getEntityTypeId() . '_' . $uid_key_name => !empty($entity->getOwner())
        ? $entity->getOwner()->id() : 0,
        'Name' => $entity->getName(),
      ],
      $entity->getEntityTypeId()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function createImage(ImageInterface $entity, array $params): bool {
    $this->setEntity($entity);

    $result = $this->ec2Service->createImage([
      'InstanceId'  => $entity->getInstanceId(),
      'Name'        => $entity->getName(),
      'Description' => $entity->getDescription(),
      'Visibility'  => $entity->getVisibility(),
    ]);

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'created remotely');
      return TRUE;
    }

    $this->cloudConfigPluginManager->setCloudContext($entity->getCloudContext());
    $account_id = $this->cloudConfigPluginManager->loadConfigEntity()->get('field_account_id')->value;

    if (
      !empty($entity) && !empty($result['ImageId'])
      && ($entity->setName($params['name']))
      && ($entity->set('ami_name', $params['name']))
      && ($entity->setImageId($result['ImageId']))
      && ($entity->set('account_id', $account_id))
      && ($entity->save())
    ) {

      $this->updateNameAndCreatedByTags($entity, $entity->getImageId());

      $this->processOperationStatus($entity, 'created');

      $this->dispatchSaveEvent($entity);

      if ($this->ec2Service instanceof OpenStackRestService) {
        $this->updateNameAndCreatedByTags($entity, $entity->getImageId());
      }

      return TRUE;
    }
    else {
      $this->processOperationErrorStatus($entity, 'created');

      if ($this->ec2Service instanceof OpenStackRestService) {
        $this->updateNameAndCreatedByTags($entity, $entity->getImageId());
      }

      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function editImage(ImageInterface $entity, array $params): bool {
    // Switch OpenStack EC2 or Rest service based on $entity->getCloudContext().
    $ec2Service = $this->openStackServiceFactory->get($entity->getCloudContext());
    $this->setEc2Service($ec2Service);

    if (!($entity instanceof CloudConfigInterface)) {
      $this->cloudConfigPluginManager->setCloudContext($entity->getCloudContext());
      $cloud_config = $this->cloudConfigPluginManager->loadConfigEntity();
    }

    if (!$entity->isNew() && !($entity instanceof CloudConfigInterface) && $cloud_config->isRemote()) {
      $status = SAVED_UPDATED;
      $passive_operation = 'updated remotely';
      $this->processOperationStatus($entity, $passive_operation);
    }
    else {
      $status = $entity->save();

      // Add a updated status message and the log.
      switch ($status) {
        case SAVED_NEW:
          $passive_operation = 'created';
          break;

        case SAVED_DELETED:
          $passive_operation = 'deleted';
          break;

        default:
        case SAVED_UPDATED:
          $passive_operation = 'updated';
      }

      if (!($entity instanceof CloudConfigInterface) && $cloud_config->isRemote()) {
        $passive_operation .= ' remotely';
      }

      $this->processOperationStatus($entity, $passive_operation);

      // Clear block and menu cache.
      $this->clearCacheValues($entity->getCacheTags());
    }

    $this->ec2Service = $this->openStackServiceFactory->get($entity->getCloudContext());

    $original_entity = $this->entityTypeManager
      ->getStorage($entity->getEntityTypeId())
      ->load($entity->id());

    $this->updateNameAndCreatedByTags($entity, $entity->getImageId());

    $this->cloudConfigPluginManager->setCloudContext($entity->getCloudContext());
    $cloud_config = $this->cloudConfigPluginManager->loadConfigEntity();

    // Call 'modifyImageAttribute'
    // only when OwnerId is same as Account ID of Cloud Config.
    // It's to avoid AuthFailure error from AWS API.
    if ($cloud_config->get('field_account_id')->value === $entity->getAccountId()) {
      $visibility = $params['visibility'] ?? 0;

      $permission = [
        [
          'Group' => 'all',
          'UserId' => $entity->getAccountId(),
        ],
      ];

      // 1 => Public, 0 => Private.
      $launch_permission =
        $visibility
          ? ['Add' => $permission]
          : ['Remove' => $permission];

      // Update image.
      $this->ec2Service->setCloudContext($entity->getCloudContext());

      if (method_exists($this->ec2Service, 'modifyImageAttribute')) {
        // Update image description.
        $this->ec2Service->modifyImageAttribute([
          'ImageId' => $entity->getImageId(),
          'Description' => ['Value' => $entity->getDescription()],
        ]);

        // These fields (Description , LaunchPermission , or ProductCode) cannot
        // be specified at the same time.
        // See also:
        // https://docs.aws.amazon.com/cli/latest/reference/ec2/modify-image-attribute.html
        // Update image launch permission.
        $this->ec2Service->modifyImageAttribute([
          'ImageId' => $entity->getImageId(),
          'LaunchPermission' => $launch_permission,
        ]);

        $new_account_ids = [];
        foreach ($entity->getLaunchPermissionAccountIds() as $account_id) {
          if (empty($account_id->value)) {
            continue;
          }
          $new_account_ids[] = $account_id->value;
        }

        $old_account_ids = [];
        foreach ($original_entity->getLaunchPermissionAccountIds() as $account_id) {
          if (empty($account_id->value)) {
            continue;
          }
          $old_account_ids[] = $account_id->value;
        }

        $account_ids_to_remove = array_diff($old_account_ids, $new_account_ids);
        $account_ids_to_add = array_diff($new_account_ids, $old_account_ids);

        if (!empty($account_ids_to_remove)) {
          $this->ec2Service->modifyImageAttribute([
            'ImageId' => $entity->getImageId(),
            'LaunchPermission' => [
              'Remove' => array_map(function ($item) {
                return ['UserId' => $item];
              }, $account_ids_to_remove),
            ],
          ]);
        }

        if (!empty($account_ids_to_add)) {
          $this->ec2Service->modifyImageAttribute([
            'ImageId' => $entity->getImageId(),
            'LaunchPermission' => [
              'Add' => array_map(function ($item) {
                return ['UserId' => $item];
              }, $account_ids_to_add),
            ],
          ]);
        }

        $this->clearCacheValues($entity->getCacheTags());
        $this->dispatchSaveEvent($entity);
      }
    }

    $visibility = !empty($entity->getVisibility()) ? 'public' : 'private';

    // Update Image of OpenStack REST API.
    if ($this->ec2Service instanceof OpenStackRestService) {
      // Make an API call to update image.
      $result = $this->ec2Service->updateImage([
        'ImageId' => $entity->getImageId(),
        'Name' => $entity->getName(),
        'Visibility' => $visibility,
      ]);

      $old_tags = $result['Tags'] ?? [];

      foreach ($old_tags ?: [] as $tag) {
        // Delete old tags.
        $this->ec2Service->deleteTags(
          [
            'ImageId'    => $entity->getImageId(),
            'Tag'        => $tag,
            'EntityType' => $entity->getEntityTypeId(),
          ]);
      }

      $this->updateNameAndCreatedByTags($entity, $entity->getImageId());
    }

    return TRUE;
  }

}
